package com.ekexiu.console.system.service;

import java.lang.reflect.Type;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicReference;

import javax.sql.DataSource;

import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Put;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.PathVar;
import org.jfw.apt.web.annotation.param.RequestBody;
import org.jfw.util.exception.JfwBaseException;
import org.jfw.util.jdbc.JdbcTask;
import org.jfw.util.jdbc.JdbcUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

import com.ekexiu.console.system.dao.DictDao;
import com.ekexiu.console.system.po.Dict;
import com.ekexiu.console.system.vo.DictInfo;
import com.ekexiu.console.system.vo.DictItem;
import com.ekexiu.console.system.vo.EditDictItem;

@Path("/sys/dict")
public class DictService {

	private final static Type LINKEDLIST_EDITDICTITEM_TYPE = new TypeReference<List<EditDictItem>>() {
	}.getType();

	private AtomicReference<Map<String, DictInfo>> dicts = new AtomicReference<Map<String, DictInfo>>();
	private Map<String, AtomicReference<LinkedList<DictItem>>> items = new HashMap<String, AtomicReference<LinkedList<DictItem>>>();

	@Autowrie
	private DictDao dictDao;

	public DictDao getDictDao() {
		return dictDao;
	}

	public void setDictDao(DictDao dictDao) {
		this.dictDao = dictDao;
	}

	@Get
	@Path("/all")
	public List<DictInfo> getAll() {
		Collection<DictInfo> ds = this.dicts.get().values();
		ArrayList<DictInfo> ret = new ArrayList<DictInfo>(ds.size());
		ret.addAll(ds);
		return ret;
	}

	@Get
	@Path("/info/{code}")
	public DictInfo get(@PathVar String code) {
		return this.dicts.get().get(code);
	}

	@Get
	@Path("/item/{code}")
	public List<DictItem> getItems(@PathVar String code) {
		AtomicReference<LinkedList<DictItem>> ref = this.items.get(code);
		return ref != null ? ref.get() : Collections.<DictItem> emptyList();
	}

	@Get
	@Path("/editItem/{code}")
	public List<EditDictItem> getItems(@JdbcConn Connection con, @PathVar String code) throws SQLException {
		DictInfo di = this.dicts.get().get(code);
		if (di == null || di.isReadonly())
			return Collections.<EditDictItem> emptyList();

		String val = this.dictDao.queryItem(con, code);
		if (val == null || val.length() < 3)
			return Collections.<EditDictItem> emptyList();
		return JsonService.fromJson(val, LINKEDLIST_EDITDICTITEM_TYPE);
	}

	@Put
	@Path("/info")
	public void update(@JdbcConn final Connection con, @RequestBody final Dict dict) throws SQLException {
		final AtomicBoolean ab = new AtomicBoolean(false);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {

			@Override
			public void exec() throws SQLException {
				ab.set(dictDao.update(con, dict) > 0);
			}
		});

		if (ab.get())
			this.update(dict);
	}

	@Put
	@Path("/item/{code}")
	public void update(@JdbcConn(true) final Connection con, @PathVar final String code, @RequestBody List<EditDictItem> items)
			throws SQLException, JfwBaseException {
		DictInfo di = this.dicts.get().get(code);
		if (di == null) {
			throw new JfwBaseException(50001, "invalid parameter code: not exists dict code");
		}
		if (di.isReadonly()) {
			throw new JfwBaseException(50002, "invalid parameter code: readonly dict code");
		}
		// TODO : check dictItem.system == true can't modify
		LinkedList<DictItem> ret = new LinkedList<DictItem>();
		if(di.isTree()){
		if (transfer(items, ret) == null) {
			throw new JfwBaseException(50003, "not exists parentCode or loop parentCode");
		}}else{
			for(ListIterator<EditDictItem> it = items.listIterator();it.hasNext();){
				ret.add(transfer(it.next()));
			}
		}
		final AtomicBoolean ab = new AtomicBoolean(false);
		final String val = JsonService.toJson(items);
		JdbcUtil.executeAutoCommit(con, new JdbcTask() {
			public void exec() throws SQLException {
				if (dictDao.update(con, code, val) > 0)
					ab.set(true);
			}
		});
		if (ab.get()) {
			this.items.get(code).set(ret);
		}
	}

	// public static List
	public DictItem queryItem(String dictCode, String itemCode) {
		AtomicReference<LinkedList<DictItem>> aitems = this.items.get(dictCode);
		if (null != aitems) {
			LinkedList<DictItem> ims = aitems.get();
			if (ims != null && ims.size() > 0) {
				return query(ims, itemCode);
			}
		}
		return null;
	}

	private DictItem query(LinkedList<DictItem> items, String code) {
		for (ListIterator<DictItem> it = items.listIterator(); it.hasNext();) {
			DictItem item = it.next();
			if (item.getCode().equals(code))
				return item;
			LinkedList<DictItem> citems = item.getChildren();
			if (citems.size() > 0) {
				item = query(citems, code);
				if (null != item)
					return item;
			}
		}
		return null;
	}

	public LinkedList<DictItem> transfer(List<EditDictItem> src, LinkedList<DictItem> dest) {
		Map<String, EditDictItem> map = new HashMap<String, EditDictItem>();
		for (EditDictItem eitem : src) {
			map.put(eitem.getCode(), eitem);
		}
		LinkedList<String> removeCodes = new LinkedList<String>();
		for (;;) {
			for (String code : removeCodes) {
				map.remove(code);
			}
			removeCodes.clear();
			for (Map.Entry<String, EditDictItem> entry : map.entrySet()) {
				String code = entry.getKey();
				EditDictItem eitem = entry.getValue();
				String pc = eitem.getParentCode();
				if (pc == null || pc.length() == 0) {
					removeCodes.add(code);
					dest.add(transfer(eitem));
				} else {
					DictItem item = query(dest, pc);
					if (item != null) {
						removeCodes.add(code);
						item.getChildren().add(transfer(eitem));
					}
				}
			}
			if (removeCodes.size() == map.size())
				return dest;
			if (removeCodes.isEmpty()) {
				return null;
			}
		}
	}

	public DictItem transfer(EditDictItem eitem) {
		DictItem item = new DictItem();
		item.setCaption(eitem.getCaption());
		item.setCode(eitem.getCode());
		item.setData(eitem.getData());
		item.setEnabled(eitem.isEnabled());
		item.setShortCode(eitem.getShortCode());
		item.setSystem(eitem.isSystem());
		return item;
	}

	private LinkedList<DictItem> transfer(String s) {
		LinkedList<DictItem> ret = new LinkedList<DictItem>();
		if (s == null || s.length() == 0 || s.equals("[]")) {
			return ret;
		}
		try {
			List<EditDictItem> list = JsonService.fromJson(s, LINKEDLIST_EDITDICTITEM_TYPE);
			if (null == transfer(list, ret)) {
				// TODO:: // log not exists parentCode or loop parentCode

			}
		} catch (Exception e) {
			// TODO: ///// log json parse exception

		}
		return ret;
	}

	public void initByDataSource(DataSource ds) throws SQLException {
		final Connection con = ds.getConnection();
		JdbcUtil.executeAutoClose(con, new JdbcTask() {
			@Override
			public void exec() throws SQLException {
				List<Dict> list = DictService.this.dictDao.queryAlll(con);
				DictService.this.init(list);
			}
		}, false);
	}

	private synchronized void init(List<Dict> list) {
		if (null != this.dicts.get())
			return;
		Map<String, DictInfo> map = new HashMap<String, DictInfo>();
		for (Dict dict : list) {
			map.put(dict.getCode(), new DictInfo(dict));
			AtomicReference<LinkedList<DictItem>> ar = new AtomicReference<LinkedList<DictItem>>(transfer(dict.getItem()));
			this.items.put(dict.getCode(), ar);
		}
		this.dicts.set(map);
	}

	private void update(Dict dict) {
		Map<String, DictInfo> map = new HashMap<String, DictInfo>();
		map.putAll(this.dicts.get());
		map.remove(dict.getCode());
		map.put(dict.getCode(), new DictInfo(dict));
		this.dicts.set(map);
	}

}
